const int lights = 2;

uniform sampler2D albedoMap;
uniform sampler2D normalMap;

varying vec3 position;
varying vec3 normal;
varying vec2 texCoord;
varying vec3 tangent;
varying vec3 binormal;
varying vec3 lightDir0;
varying vec3 lightDir1;
varying vec3 halfVec0;
varying vec3 halfVec1;

vec3 rgbToXyz( vec3 rgb ) {
	return (rgb * 2.0f) - 1.0f;
}

void main() {
	vec3 lightDirV[lights];
	vec3 halfVecV[lights];

	lightDirV[0] = lightDir0;
	lightDirV[1] = lightDir1;
	halfVecV[0] = lightDir0;
	halfVecV[1] = lightDir1;

	// get normal, tangent, binormal in eye space
	vec3 nNormal = normalize( normal );
	vec3 nTangent = normalize( tangent );
	vec3 nBinormal = normalize( binormal );

	// modify the normal using the normal map
	vec3 normalDir = rgbToXyz( texture2D( normalMap, texCoord ).xyz );
	nNormal = normalize( normalDir.x*nTangent + normalDir.y*nBinormal + normalDir.z*nNormal );

	// add constant values
	vec4 intensity = gl_FrontMaterial.emission;

	// loop through each light
	for (int i = 0; i < lights; ++i) {
		// add ambient
		intensity += gl_LightSource[i].ambient * gl_FrontMaterial.ambient;

		// attenuation
		float dist = length( gl_LightSource[i].position.xyz - position );
		float att = 1.0f/(gl_LightSource[i].constantAttenuation +
						  gl_LightSource[i].linearAttenuation * dist +
						  gl_LightSource[i].quadraticAttenuation * dist * dist);

		// add diffuse
		float nDotL = max( dot( nNormal, normalize( lightDirV[i] ) ), 0.0f );
		intensity += att * gl_LightSource[i].diffuse * gl_FrontMaterial.diffuse * nDotL;

		// add specular - clamp gets rid of specular on backfaces using a smooth transition
		float nDotH = max( dot( nNormal, normalize( halfVecV[i] ) ), 0.0f );
		float specular = pow( nDotH, gl_FrontMaterial.shininess ) * clamp( nDotL*5.0f, 0.0f, 1.0f );
		intensity += att * gl_LightSource[i].specular * gl_FrontMaterial.specular * specular;
	}

	gl_FragColor = texture2D( albedoMap, texCoord ) * intensity;
}