const int lights = 2;

attribute vec3 inPosition;
attribute vec3 inNormal;
attribute vec2 inTexCoord;
attribute vec3 inTangent;
attribute vec3 inBinormal;

varying vec3 position;
varying vec3 normal;
varying vec2 texCoord;
varying vec3 tangent;
varying vec3 binormal;
varying vec3 lightDir0;
varying vec3 lightDir1;
varying vec3 halfVec0;
varying vec3 halfVec1;

void main() {
	vec3 lightDirV[2]; // light dir array
	vec3 halfVecV[2]; // half vec array

	// compute the position in eye space
	position = vec3( gl_ModelViewMatrix * vec4( inPosition, 1 ) );

	// compute the normal, tangent, and binormal in eye space
	normal = normalize( gl_NormalMatrix * inNormal );
	tangent = normalize( gl_NormalMatrix * inTangent );
	binormal = normalize( gl_NormalMatrix * inBinormal );

	// interpolate texture coordinate
	texCoord = inTexCoord;

	for (int i = 0; i < lights; ++i) {
		// compute the direction to the light in eye space
		lightDirV[i] = normalize( gl_LightSource[i].position.xyz - position );

		// compute the halfway vector
		vec3 eyeDir = -normalize( position );
		halfVecV[i] = normalize( (lightDirV[i] + eyeDir)/2.0f );
	}

	lightDir0 = lightDirV[0];
	lightDir1 = lightDirV[1];
	halfVec0 = halfVecV[0];
	halfVec1 = halfVecV[1];

	gl_Position = gl_ModelViewProjectionMatrix * vec4( inPosition, 1.0f );
}